%{
/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.3 04/19/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 

#include <strings.h>
#include <syslog.h>

#include "vtun.h"

#include "cfg_file.tab.h"
#include "cfg_file.h"

/* static buffer for strings */
char str_buf[255];

int find_keyword(struct kword *kw, char *str);

#define ECHO {;}
#define YY_DECL int yylex(void)

int yyerror(char *str);
%}

num		[0-9]+
dnum		{num}:{num}
word		[A-Za-z0-9\-\_+=\!\$\#\%\&\*\^\@\\]+
keyword		[a-z_-]+	
comment		\#.*\n
ipaddr		{num}\.{num}\.{num}\.{num}
nmaddr		{word}(\.|{word})*
path		(\/{word})+
string		\".*\"

%x PARAM 

%%
{comment}	{ lineno++; } /* skip comments */

{keyword}	{
		  int kw = find_keyword(cfg_keywords,yytext);
		  yylval.str = yytext;

		  if( kw != -1 ){	
		     /* K_DEFAULT & K_OPTIONS is _not_ host options
		      * don't begin PARAM stage for them */
		     if(kw != K_DEFAULT && kw != K_OPTIONS) BEGIN(PARAM);
		     return kw;
		  }
		  /* Keyword not found, means host definition. */
		  return K_HOST;
		}

<PARAM>{string}	{
		  if(yyleng > sizeof(str_buf)-1){
		     yyerror("string to long");
		     return 0;
		  }
		  strncpy(str_buf, yytext+1, yyleng-2);
		  str_buf[yyleng-2] = '\0';

		  yylval.str = str_buf;
					  
		  return STRING;
		}

<PARAM>{dnum}   {
	          char *ptr = strchr(yytext,':') + 1;
		 
		  yylval.dnum.num1 = atoi(yytext);
		  yylval.dnum.num2 = atoi(ptr);
		  return DNUM;
		}

<PARAM>{num}	{
		  yylval.num = atoi(yytext);
		  return NUM;
		}

<PARAM>{word}	{
		  int kw = find_keyword(cfg_param,yytext);

		  if( kw != -1 ){	
		     /* Special parametr(yes,no) found,
		      * return its numeric value. */ 		
		     yylval.num = kw;
		     return NUM;
		  }

		  yylval.str = yytext;
		  return WORD;
		}

<PARAM>{ipaddr}	{
		  yylval.str = yytext;
		  return IPADDR;
		}

<PARAM>{nmaddr}	{
		  yylval.str = yytext;
		  return NMADDR;
		}

<PARAM>{path}	{
		  yylval.str = yytext;
		  return PATH;
		}

<PARAM>\;	BEGIN(0);

\n		{ lineno++; } 

[ \t]		; /* skip whitespace */

.		return *yytext;

%%

int yywrap(void) 
{
    return 1;
}


int find_keyword(struct kword *kw, char *str)
{
    while( kw->str ){
	if( !strncmp(str,kw->str,4) )
	   return kw->type;
	kw++;
    }
    return -1;
}




