/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.3 04/19/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <syslog.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>
#include <resolv.h>
#include <netdb.h>

#include "vtun.h"
#include "lib.h"

static int auth(int fd, struct vtun_host * host);

void client(struct vtun_host *host, char *svr)
{
     struct sockaddr_in my_addr,svr_addr;
     struct hostent *hent;
     struct sigaction sa;
     int s;	

     /* 
      * Clear speed and flags, they will be provided 
      * by the server. 
      */
     host->spd_in = host->spd_out = 0;
     host->flags  = 0;

     if( !(hent = gethostbyname(svr)) ){
	  syslog(LOG_ERR,"Can't resolve server %s IP address",svr);
	  exit(1);
     }

     memset(&sa,0,sizeof(sa));     
     sa.sa_handler=SIG_IGN;
     sigaction(SIGHUP,&sa,NULL);
     sigaction(SIGINT,&sa,NULL);
     sigaction(SIGQUIT,&sa,NULL);
     sigaction(SIGCHLD,&sa,NULL);
     sigaction(SIGPIPE,&sa,NULL);

     memset(&my_addr,0,sizeof(my_addr));
     my_addr.sin_family = AF_INET;
     my_addr.sin_addr.s_addr = INADDR_ANY;
     my_addr.sin_port = 0;	

     memset(&svr_addr,0,sizeof(my_addr));
     svr_addr.sin_family = AF_INET;
     svr_addr.sin_addr.s_addr = *((unsigned long *)hent->h_addr);
     svr_addr.sin_port = htons(vtun.port);	

     while(1){ 
	/* We have to create socket again each time
	 * we want to connect, since STREAM sockets 
	 * can be successfuly connected only once.
	 */
	
        if( (s=socket(AF_INET,SOCK_STREAM,0))==-1 ){
	  syslog(LOG_ERR,"Can not create socket");
	  exit(1);
        }

        if( bind(s,(struct sockaddr *)&my_addr,sizeof(my_addr)) ){
	  syslog(LOG_ERR,"Can not bind to the socket");
	  exit(1);
        }

	set_title("connecting to %s",svr);

        if( connect_t(s,(struct sockaddr *) &svr_addr, vtun.timeout) ){
	  syslog(LOG_INFO,"Can't connect to the server %s",svr);
        } else {
	  if( auth(s, host) ){   
	     syslog(LOG_INFO,"Session %s[%s] opened",host->host,svr);

 	     host->rmt_fd = s;
	     start_tunnel(host);
	     /* rmt_fd might be changed in case of UDP proto */	
 	     s = host->rmt_fd;

	     syslog(LOG_INFO,"Session %s[%s] closed",host->host,svr);
	  } else {
	     syslog(LOG_INFO,"Connection dennied by server",svr);
	  }
	}

	close(s);
	
	/* If persist option is set, sleep and try to reconnect */
	if(!vtun.persist) 
	  break;
        else
	  sleep(5);
     }
     syslog(LOG_INFO,"Exit");
     return;
}
	
#define ST_INIT  0
#define ST_HOST  1
#define ST_PASS  2

static int auth(int fd, struct vtun_host *host)
{
	int stage, success=0 ;
	char buf[VTUN_MESG_SIZE];
	
	stage = ST_INIT;

	while( read_t(fd,buf,sizeof(buf),VTUN_TIMEOUT) > 0 ){
	   buf[sizeof(buf)-1]='\0';
	   switch(stage){
		case ST_INIT:
	    	   if( !strncmp(buf,"VTUN",4) ){
		      stage = ST_HOST;
		      print_p(fd,"HOST: %s\n",host->host);
		      continue;
	           }
		   break;	

	        case ST_HOST:
		   if( !strncmp(buf,"OK",2) ){
		      stage = ST_PASS;
		      print_p(fd,"PASS: %s\n",host->passwd);
		      continue;
	   	   }
		   break;	
	
	        case ST_PASS:
		   if( !strncmp(buf,"OK",2) && cf2bf(buf,host) )
		      success = 1;
		   break;
	   }
	   break;
	}

	return success;
}
