/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.3 04/19/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <unistd.h>
#include <stdlib.h>
#include <stdio.h>
#include <syslog.h>
#include <strings.h>

#include "vtun.h"
#include "linkfd.h"
#include "lib.h"

/* 
 * Encryptor module. 
 * It is _NOT_ realy good encryption, it just a simple 
 * example how to write linkfd module.
 */

char *xor_msk;
int  xor_len;

/* 
 * Initialization function.
 * It have to allocate buffers(if needed), and performs 
 * all required initialization. Also it can do any handshaking
 * with other end over host->rmt_fd descriptor. 
 */
int alloc_encrypt(struct vtun_host *host)
{
     /* Encrypt traffic using host->passwd */
     if( !(xor_msk = strdup(host->passwd)) )
        return 1;

     xor_len  = strlen(xor_msk);

     return 0;
}

/* 
 * Deinitialization function.
 * It have to free buffers(if needed), and performs 
 * all required deinitialization. Also it can do any handshaking
 * with other end over host->rmt_fd descriptor. 
 */
int free_encrypt()
{
     free(xor_msk);	

     return 0;
}

/* 
 * Encryption function.
 * It is called when we run modules list DOWN, and performs 
 * required encryption of the 'in' buffer and stores resulting 
 * buffer address in the 'out' pointer. 
 * This function _MUST_ consume all incoming bytes in _ONE_ pass.
 */
int encrypt_buf(int len, char *in, char **out)
{ 
     static int xor_pos = 0;
     register int i;
 
     for(i = 0; i < len; i++){
        in[i] ^= xor_msk[xor_pos++];
	if(xor_pos > xor_len)
	   xor_pos = 0;
     } 

     *out = in;
     return len;
}

/* 
 * This function is called when we run modules list DOWN to check 
 * if the module is accepting the data. 
 * This function _MUST_ return 1 if module is able to handle the data
 * and 0 otherwise.
 */
int encrypt_avail(void)
{ 
     /* We are always ready */	
     return 1;
}

/* 
 * Decryption function.
 * It is called when we run modules list UP, and performs 
 * required decryption of the 'in' buffer and store resulting 
 * buffer address in the 'out' pointer. 
 * This function _MUST_ consume all incoming bytes in _ONE_ pass.
 */
int decrypt_buf(int len, char *in, char **out)
{
     static int xor_pos = 0;
     register int i;
 
     for(i = 0; i < len; i++){
        in[i] ^= xor_msk[xor_pos++];
	if(xor_pos > xor_len)
	   xor_pos = 0;
     } 

     *out = in;
     return len;
}

/* 
 * This function is called when we run modules list UP to check 
 * if the module is accepting the data. 
 * This function _MUST_ return 1 if module is able to handle the data
 * and 0 otherwise.
 */
int decrypt_avail(void)
{ 
     /* We are always ready */	
     return 1;
}

/* 
 * Module structure.
 */
struct lfd_mod lfd_encrypt = {
     "Encryptor",
     alloc_encrypt,
     encrypt_buf,
     encrypt_avail,
     decrypt_buf,
     decrypt_avail,
     free_encrypt,
     NULL,NULL
};
