/*  
    VTun - Virtual Tunnel over TCP/IP network. 

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.3 04/19/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <unistd.h>
#include <stdlib.h>
#include <syslog.h>
#include <sys/time.h>

#include "vtun.h"
#include "linkfd.h"
#include "lib.h"

/* 
 * Shaper module. 
 */

unsigned long bytes;
float max_speed;
struct timeval curr_time, last_time;

/* 
 * Initialization function.
 */
int shaper_alloc(struct vtun_host *host)
{
     /* Calculate max speed bytes/sec */
     max_speed = (float)host->spd_out / 8.0 * 1024.0;
   
     /* Compensation for delays, nanosleep and so on */ 
     max_speed += 400;

     bytes = 0;

     return 0;
}

int shaper_free()
{
     return 0;
}

/* Shaper counter */
int shaper_counter(int len, char *in, char **out)
{ 
     /* Just count incoming bytes */
     bytes += len;

     *out = in;
     return len;
}

/* Convert tv stuct to floating point number */
float inline tv2fl(struct timeval tv)
{
     return (float)tv.tv_sec + (float)(tv.tv_usec/1000000.0); 
}

/* 
 * Main shaper function.
 * Compute current speed in bytes/sec and if it is
 * higher than maximal speed stop accepting input 
 * until the speed become lower or equal to maximal.
 */
int shaper_avail(void)
{ 
     static struct timeval tv;
     static float speed;

     /* Let me know if you have faster and better time source. */
     gettimeofday(&curr_time,NULL);

     timersub(&curr_time,&last_time,&tv);

     /* Floating point ? Why not we are in the User Space */ 
     speed = (float)bytes / tv2fl(tv); 
	
     if( speed > max_speed ){
	/* 
	 * Sleep about 1 microsec(actual sleep might be longer). 
	 * This is actualy the hack to reduce CPU usage. 
	 * Without this delay we will get 100% CPU.
         */ 
        static struct timespec ts = {0,1000};
     	nanosleep(&ts,NULL);

	/* Don't accept input */
	return 0;
     }

     if( curr_time.tv_sec > last_time.tv_sec ){
        last_time = curr_time;
        bytes = 0;
     }

     /* Accept input */
     return  1;
}

struct lfd_mod lfd_shaper = {
     "Shaper",
     shaper_alloc,
     shaper_counter,
     shaper_avail,
     NULL,
     NULL,
     shaper_free,
     NULL,NULL
};
