/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.3 04/19/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#include <stdio.h>
#include <strings.h>
#include <stdlib.h>
#include <unistd.h>
#include <signal.h>
#include <fcntl.h>
#include <signal.h>
#include <syslog.h>

#include <netinet/in.h>
#include <netinet/tcp.h>
#include <arpa/inet.h>

#include "vtun.h"
#include "lib.h"

static struct vtun_host * auth(int fd);

void server(void)
{
     struct sockaddr_in my_addr,cl_addr;
     struct sigaction sa;
     struct vtun_host *host;
     int  s,s1;
     int  opt;
     char *ip;

     sa.sa_handler=SIG_IGN;
     sigaction(SIGINT,&sa,NULL);
     sigaction(SIGQUIT,&sa,NULL);
     sigaction(SIGCHLD,&sa,NULL);
     sigaction(SIGPIPE,&sa,NULL);

     memset(&my_addr,0,sizeof(my_addr));
     my_addr.sin_family = AF_INET;
     my_addr.sin_addr.s_addr = htonl(INADDR_ANY);
     my_addr.sin_port = htons(vtun.port);	

     if( (s=socket(AF_INET,SOCK_STREAM,0))== -1 ){
	syslog(LOG_ERR,"Can't creat socket");
	exit(1);
     }

     opt=1;
     setsockopt(s, SOL_SOCKET, SO_REUSEADDR, &opt, sizeof(opt)); 

     if( bind(s,(struct sockaddr *)&my_addr,sizeof(my_addr)) ){
	syslog(LOG_ERR,"Can't bind to the socket");
	exit(1);
     }

     if( listen(s,5) ){
	syslog(LOG_ERR,"Can't listen on the socket");
	exit(1);
     }

     syslog(LOG_INFO,"VTUN server ver %s started ",VER);

     set_title("waiting for connections on port %d",vtun.port);

     while(1){
        opt=sizeof(cl_addr);
	if( (s1=accept(s,&cl_addr,&opt)) == -1 )
	   continue; 

	if( !fork() ){
	   ip=inet_ntoa(cl_addr.sin_addr);
	   if( (host=auth(s1)) ){	
              sa.sa_handler=SIG_IGN;
              sigaction(SIGHUP,&sa,NULL);

	      syslog(LOG_INFO,"Session %s[%s] opened ", host->host, ip);

              host->rmt_fd = s1; 
	      start_tunnel(host);
	      /* rmt_fd might be changed in case of UDP proto */
	      s1 = host->rmt_fd; 

	      syslog(LOG_INFO,"Session %s[%s] closed", host->host, ip);
	   } else {
	      syslog(LOG_INFO,"Denied connection from %s", ip);
	   }
	
	   close(s1);
	   exit(0);
        }
	close(s1);
     }  
}	

#define ST_HOST  1
#define ST_PASS  2

static struct vtun_host * auth(int fd)
{
        struct vtun_host *h = NULL;
	char buf[VTUN_MESG_SIZE],*str1,*str2;
	char host[9],pass[9];	
	int stage;

        set_title("authentification");

	print_p(fd,"VTUN server ver %s\n",VER);

	stage = ST_HOST;

	while( read_t(fd,buf,sizeof(buf),VTUN_TIMEOUT) > 0 ){
	   buf[sizeof(buf)-1]='\0';
	   strtok(buf,"\r\n");

	   if( !(str1=strtok(buf," :")) )
		break;
	   if( !(str2=strtok(NULL," :")) )
		break;

	   switch(stage){
	     case ST_HOST:
	        if( !strcmp(str1,"HOST") ){
		   strncpy(host,str2,sizeof(host));
		   host[sizeof(host)-1]='\0';
		   stage = ST_PASS;
		   print_p(fd,"OK\n");
		   continue;
	        }
		break;
	     case ST_PASS:
	        if( !strcmp(str1,"PASS") ){
		   strncpy(pass,str2,sizeof(pass));
		   pass[sizeof(pass)-1]='\0';
		   if( (h = find_host(host,pass)) )
		      print_p(fd,"OK FLAGS: %s\n", bf2cf(h)); 
	        }
		break;
 	   }
	   break;
	}

	if(!h)
	  print_p(fd,"ERR\n");	

	return h;
}
