/*  
    VTun - Virtual Tunnel over TCP/IP network.

    Copyright (C) 1998,1999  Maxim Krasnyansky <max_mk@yahoo.com>

    VTun has been derived from VPPP package by Maxim Krasnyansky. 

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 */

/*
 * Version: 1.3 04/19/1999 Maxim Krasnyansky <max_mk@yahoo.com>
 */ 
#ifndef _VTUN_H
#define _VTUN_H

#define VER "1.3"

/* Default VTUN port */
#define VTUN_PORT 5000

/* Default VTUN connect timeout in sec */
#define VTUN_CONNECT_TIMEOUT 30

/* General VTUN timeout for several operations, in sec */
#define VTUN_TIMEOUT 30

/* Default config file */
#define VTUN_CONFIG_FILE "/etc/vtund.conf"

/* PID file */
#define VTUN_PID_FILE "/var/run/vtund.pid"

/* End of configurable part */

struct vtun_host {
   char *host;
   char *passwd;
   char *prog;
   char *args;
   int  spd_in;
   int  spd_out;
   int  flags;

   int  rmt_fd;
   int  loc_fd;

   struct vtun_host *next;
};

/* Flags definitions */
#define VTUN_TTY       0x1000
#define VTUN_PIPE      0x2000
#define VTUN_ETHER     0x4000
#define VTUN_TYPE_MASK (VTUN_TTY | VTUN_PIPE | VTUN_ETHER) 

#define VTUN_TCP       0x0100  
#define VTUN_UDP       0x0200  
#define VTUN_PROT_MASK (VTUN_TCP | VTUN_UDP) 

#define VTUN_COMPRESS  0x0001
#define VTUN_SHAPE     0x0002
#define VTUN_ENCRYPT   0x0004

extern struct vtun_host * host_list;

/* Authentification message size */
#define VTUN_MESG_SIZE 40

/* Global options */
struct vtun_opts {
   int server;
   int daemon;
   int port;
   int persist;
   int timeout;
   char *cfg_file;
};

extern struct vtun_opts vtun;

struct vtun_host * find_host(char *host, char *pwd);
struct vtun_host * read_config(char *file, char *host);

void server(void);
void client(struct vtun_host *host, char * svr);
#endif
